package java3.lang3;

public final class Integer extends Number implements Comparable {

	public static final int MAX_VALUE = 2147483647;

	private/*@ spec_public @*/final int intValue;

	/*@ public normal_behavior
	  @   assignable this.intValue;
	  @   ensures this.intValue == new_int_value;
	  @*/
	//@ pure
	public Integer(int new_int_value) {
		this.intValue = new_int_value;
	}

	/*@ also
	  @   public normal_behavior
	  @     ensures \result == (byte) this.intValue;
	  @*/
	//@ pure
	public byte byteValue() {
		return (byte) this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @     ensures \result == (short) this.intValue;
	  @*/
	public short shortValue() {
		return (short) this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @     ensures \result == this.intValue;
	  @*/
	public/*@ pure @*/int intValue() {
		return this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @     ensures \result == (long) this.intValue;
	  @*/
	public long longValue() {
		return (long) this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @      ensures \result == (float) this.intValue;
	  @*/
	public float floatValue() {
		return (float) this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @     ensures \result == (double) this.intValue;
	  @*/
	public double doubleValue() {
		return (double) this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @     ensures \result == this.intValue;
	  @*/
	public int hashCode() {
		return this.intValue;
	}

	/*@ also
	  @   public normal_behavior
	  @     requires obj != null && (obj instanceof java3.lang3.Integer);
	  @     ensures \result <==> this.intValue() == ((java3.lang3.Integer) obj).intValue();
	  @   also
	  @   public normal_behavior
	  @     requires obj == null || !(obj instanceof java3.lang3.Integer);
	  @     ensures !\result;
	  @*/
	public boolean equals(/*@nullable*/Object obj) {
		if (obj == null || !(obj instanceof java3.lang3.Integer))
			return false;
		return this.intValue() == ((java3.lang3.Integer) obj).intValue();
	}

	/*@ public normal_behavior
	  @ requires anotherInteger != null;
	  @ {|
	  @   requires this.intValue == anotherInteger.intValue();
	  @   ensures \result == 0;
	  @ also 
	  @   requires this.intValue < anotherInteger.intValue();
	  @   ensures \result == -1;
	  @ also
	  @   requires this.intValue > anotherInteger.intValue();
	  @   ensures \result == 1;
	  @ |}
	  @ also public exceptional_behavior
	  @   requires anotherInteger == null;
	  @   signals_only NullPointerException;
	  @*/
	public int compareTo(java3.lang3.Integer anotherInteger) {
		if (anotherInteger == null)
			throw new NullPointerException();
		if (this.intValue() == anotherInteger.intValue())
			return 0;
		if (this.intValue() < anotherInteger.intValue())
			return -1;
		if (this.intValue() > anotherInteger.intValue())
			return 1;
		// Unreacheable, but the compiler demands it.
		return 0;
	}

	/*@ also
	  @   public normal_behavior
	  @     requires o != null && (o instanceof java3.lang3.Integer);
	  @     ensures \result == compareTo((java3.lang3.Integer) o);
	  @ also
	  @   public exceptional_behavior
	  @     requires o != null && !(o instanceof java3.lang3.Integer);
	  @     signals_only ClassCastException;
	  @ also public exceptional_behavior
	  @   requires o == null;
	  @   signals_only NullPointerException;
	  @*/
	public int compareTo(Object o) {
		if (o == null)
			throw new NullPointerException();
		if (!(o instanceof java3.lang3.Integer))
			throw new ClassCastException();
		return this.compareTo((java3.lang3.Integer) o);
	}

	/**
	 * The only purpose of this method is allowing 
	 * Chicory to access the private fields of this class.
	 * 
	 */
	private void unreachable_method() {
	}

	/*@ 
	@   public normal_behavior
	@     ensures \result == this.intValue;
	@*/
	public/*@pure@*/int hashCode0() {
		return this.intValue;
	}

}
